@@ -2,18 +2,18 @@
  * Copyright (C) 2006, Edward Duffy <eduffy@gmail.com>
  * Copyright (C) 2008, Nokia <ivan.frade@nokia.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
@@ -84,7 +84,7 @@ extract_totem (const gchar          *uri,
 	                                      (GDestroyNotify) g_free,
 	                                      (GDestroyNotify) g_free);
 
-	if (tracker_spawn (argv, 10, &totem, NULL)) {
+	if (tracker_spawn (argv, 10, &totem, NULL, NULL)) {
 		gchar **lines, **line;
 		gchar *artist = NULL, *album = NULL;
 		gchar *artist_uri = NULL, *album_uri = NULL;
@@ -141,6 +141,9 @@ extract_totem (const gchar          *uri,
 			tracker_sparql_builder_subject_iri (preupdate, album_uri);
 			tracker_sparql_builder_predicate (preupdate, "a");
 			tracker_sparql_builder_object (preupdate, "nmm:MusicAlbum");
+			/* FIXME: nmm:albumTitle is now deprecated
+			 * tracker_sparql_builder_predicate (preupdate, "nie:title");
+			 */
 			tracker_sparql_builder_predicate (preupdate, "nmm:albumTitle");
 			tracker_sparql_builder_object_unvalidated (preupdate, album);
 
