@@ -17,11 +17,12 @@
  * Boston, MA  02110-1301, USA.
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 
 #include <libtracker-sparql/tracker-sparql.h>
-#include <libtracker-common/tracker-common.h>
 
 /* This MUST be larger than TRACKER_STEROIDS_BUFFER_SIZE */
 #define LONG_NAME_SIZE 128 * 1024 * sizeof(char)
@@ -678,8 +679,8 @@ main (gint argc, gchar **argv)
 	g_type_init ();
 	g_test_init (&argc, &argv, NULL);
 
-	/* do not require prior installation */
-	g_setenv ("TRACKER_SPARQL_MODULE_PATH", "../../src/libtracker-bus/.libs", TRUE);
+	/* test D-Bus backend */
+	g_setenv ("TRACKER_SPARQL_BACKEND", "bus", TRUE);
 
 	connection = tracker_sparql_connection_get (NULL, NULL);
 
